-- $Id: DualSpeccer.lua 15 2010-09-05 01:16:42Z jeffrey0 $
local addon = LibStub("AceAddon-3.0"):NewAddon("DualSpeccer", "AceConsole-3.0", "AceEvent-3.0", "AceTimer-3.0")
DualSpeccer = addon

local groupNames = {"Primary", "Secondary"}

local primarySpecSpellName = GetSpellInfo(TALENT_ACTIVATION_SPELLS[1])
local secondarySpecSpellName = GetSpellInfo(TALENT_ACTIVATION_SPELLS[2])

local activeGroup = nil
local specCache = {}

local filtering = false
local lastSwitchStartTime = 0

local rightClickMenu = {
	{
		text = "Hide switch spam",
		func = function() addon.db.profile.filterSpam = not addon.db.profile.filterSpam; addon:Update() end,
		checked = function() return addon.db.profile.filterSpam end,
		tooltipTitle = "Hide switch spam",
		tooltipText = "Hides the spell learned/unlearned messages when swapping."
	},
	{
		text = "Show specialization name",
		func = function() addon.db.profile.showSpecName = not addon.db.profile.showSpecName; addon:Update() end,
		checked = function() return addon.db.profile.showSpecName end,
		tooltipTitle = "Show specialization name",
		tooltipText = "Displays the name of the specialization instead of primary/secondary."
	},
	{
		text = "Give custom name",
		func = function() StaticPopup_Show("DualSpeccerCustomName") end,
		tooltipTitle = "Give custom name",
		tooltipText = "Allows you to give a custom name to your specialization."
	},
}

-- Match patterns we use. Replace any "." with "%." and any "%s" with "(.*)".
local spamFilterMatch1 = string.gsub(ERR_LEARN_ABILITY_S:gsub('%.', '%.'), '%%s', '(.*)')
local spamFilterMatch2 = string.gsub(ERR_LEARN_SPELL_S:gsub('%.', '%.'), '%%s', '(.*)')
local spamFilterMatch3 = string.gsub(ERR_SPELL_UNLEARNED_S:gsub('%.', '%.'), '%%s', '(.*)')

-- Upvalues (we call FrameXML for most of the work and it's LoD)
local GetNumTalentGroups, GetActiveTalentGroup = GetNumTalentGroups, GetActiveTalentGroup

local feed = LibStub("LibDataBroker-1.1"):NewDataObject("DualSpeccer", {
	type = "data source",
	text = "DualSpeccer",
	icon = "Interface\\Icons\\Ability_Marksmanship",
	tocname = "DualSpeccer"
})

function addon:OnInitialize()
	local dbDefaults = {
		profile = {
			filterSpam = true,
			showSpecName = false
		}
	}
	self.db = LibStub("AceDB-3.0"):New("DualSpeccerDB", dbDefaults, "Default")
	self.db.char.nameOverrides = self.db.char.nameOverrides or {}
	
	StaticPopupDialogs["DualSpeccerCustomName"] = {
		hasEditBox = 1,
		text = "Enter a custom name for your active specialization:",
		button1 = ACCEPT,
		button2 = CANCEL,
		timeout = 0,
		whileDead = 1,
		hideOnEscape = 1,
		maxLetters = 20,
		OnShow = function(self)
			self.editBox:SetText(addon.db.char.nameOverrides[activeGroup] or "")
			self.editBox:SetFocus()
		end,
		OnHide = function(self)
			self.editBox:SetText("")
		end,
		OnAccept = function(self)
			local text = self.editBox:GetText()
			if text == "" then text = nil end
			addon.db.char.nameOverrides[activeGroup] = text
			self:Hide()
			addon:Update()
		end,
		EditBoxOnEnterPressed = function(self)
			local text = self:GetParent().editBox:GetText()
			if text == "" then text = nil end
			addon.db.char.nameOverrides[activeGroup] = text
			self:GetParent():Hide()
			addon:Update()
		end,
		EditBoxOnEscapePressed = function(self)
			self:GetParent():Hide()
		end,
	}
end

function addon:OnEnable()
	self:RegisterEvent("UNIT_SPELLCAST_START")
	self:RegisterEvent("ACTIVE_TALENT_GROUP_CHANGED")
	self:RegisterEvent("PLAYER_LEAVING_WORLD")
	
	-- Chances are good that the P_E_W will fire after we're done, registering PLAYER_TALENT_UPDATE twice. This
	-- is more of an AceAddon implementation thing and CallbackHandler overwrites it anyway.
	self:RegisterEvent("PLAYER_ENTERING_WORLD")
	self:RegisterEvent("PLAYER_TALENT_UPDATE")
	
	-- The important part here is that it happens after the first OnUpdate which is seemingly the time when correct
	-- information becomes available.
	self:ScheduleTimer("Update", 0.01)
end

-- A "corrupted" talent update fires between LEAVING and ENTERING when zoning. We don't want this one.
function addon:PLAYER_LEAVING_WORLD(event)
	self:UnregisterEvent("PLAYER_TALENT_UPDATE")
end

function addon:PLAYER_ENTERING_WORLD(event)
	self:RegisterEvent("PLAYER_TALENT_UPDATE")
end

function addon:ACTIVE_TALENT_GROUP_CHANGED(event, newGroup)
	activeGroup = newGroup
	local name = specCache[newGroup].specName or "unknown"
	self:Print(("Switched to %s."):format(name))
end

function addon:UNIT_SPELLCAST_START(event, ...)
	local unit, spellName = ...
	if unit == "player" and (spellName == primarySpecSpellName or spellName == secondarySpecSpellName) then
		lastSwitchStartTime = GetTime()
	end
end

function addon:PLAYER_TALENT_UPDATE(event)
	self:Update()
end

-- Our chat filter function.
local function filter(self, event, msg)
	-- You have unlearned Blahblah.
	-- You have learned a new spell: Blahblah.
	-- You have learned a new ability: Blahblah.
	if (lastSwitchStartTime > GetTime() - 8) and (msg:find(spamFilterMatch1) ~= nil or msg:find(spamFilterMatch2) ~= nil or msg:find(spamFilterMatch3) ~= nil) then
		return true
	end
end

-- Updates our talent information.
function addon:Update()
	for i = 1, GetNumTalentGroups() do
		specCache[i] = specCache[i] or {}
		local thisCache = specCache[i]
		-- Let the Blizzard code do all of the dirty work. This also ensures that it's consistent with whatever Blizzard says.
		TalentFrame_UpdateSpecInfoCache(thisCache, false, false, i)
		if thisCache.primaryTabIndex and thisCache.primaryTabIndex ~= 0 then
			thisCache.specName = thisCache[thisCache.primaryTabIndex].name
			thisCache.mainTabIcon = thisCache[thisCache.primaryTabIndex].icon
		else
			if thisCache.totalPointsSpent == 0 then
				thisCache.specName = "None"
			else
				thisCache.specName = "Hybrid"
			end
			thisCache.mainTabIcon = TALENT_HYBRID_ICON
		end
		thisCache.specGroupName = groupNames[i]
		
		-- Apply it here so we don't have to do it everywhere.
		if self.db.char.nameOverrides[i] then
			thisCache.specName = self.db.char.nameOverrides[i]
		end
	end
	
	activeGroup = GetActiveTalentGroup()
	local curCache = specCache[activeGroup]
	
	if self.db.profile.showSpecName then
		feed.text = curCache.specName
	else
		feed.text = curCache.specGroupName
	end
	
	feed.icon = curCache.mainTabIcon
	
	if not filtering and self.db.profile.filterSpam then
		ChatFrame_AddMessageEventFilter("CHAT_MSG_SYSTEM", filter)
	elseif filtering and not self.db.profile.filterSpam then
		ChatFrame_RemoveMessageEventFilter("CHAT_MSG_SYSTEM", filter)
	end
end

-- Menu anchoring stuff stolen from ClosetGnome.
local menuPopupFrame = CreateFrame("Frame", "DualSpeccerMenu", UIParent, "UIDropDownMenuTemplate")
function feed.OnClick(displayFrame, button)
	button = button or "LeftButton"
	
	if button == "LeftButton" then
		if GetNumTalentGroups() < 2 then
			addon:Print("This character does not have a second specialization.")
			return
		end
		
		-- -1 + 3 = 2 and -2 + 3 = 1.
		local newSpec = -activeGroup + 3
		SetActiveTalentGroup(newSpec)
	elseif button == "RightButton" then
		EasyMenu(rightClickMenu, menuPopupFrame, displayFrame, 20, 4, "MENU")
	end
end

function feed.OnTooltipShow(tooltip)
	tooltip:AddLine("DualSpeccer")
	local curCache = specCache[curGroup]
	
	if not curCache then
		addon:Update()
		curCache = specCache[activeGroup]
		if not curCache then
			return
		end
	end
	
	tooltip:AddDoubleLine("Active", ("%s (%s)"):format(curCache.specGroupName, curCache.specName))
	
	-- Make sure we have some valid cache data.
	if not specCache[1][1] or specCache[1][1].pointsSpent == nil then
		return
	end
	
	for i = 1, GetNumTalentGroups() do
		local specColor = (activeGroup == i) and "6adb54" or "ffff99"
		tooltip:AddDoubleLine(groupNames[i], ("|cff%s%s|r (%d/%d/%d)"):format(specColor, specCache[i].specName, specCache[i][1].pointsSpent, specCache[i][2].pointsSpent, specCache[i][3].pointsSpent))
	end
	
	if GetNumTalentGroups() >= 2 then
		tooltip:AddLine(" ")
		tooltip:AddLine("|cffeda55fClick|r to switch to your other specialization.")
	end
	
	tooltip:AddLine("|cffeda55fRight-Click|r to open the menu.")
end

